@@ -15,7 +15,6 @@ If an unknown command or parameter combination is detected, an error is
 produced.
 """
 
-from __future__ import absolute_import
 
 import getopt
 import re
@@ -59,10 +58,10 @@ def convert(s):
     helpbasic=True,
 )
 def githelp(ui, repo, *args, **kwargs):
-    '''suggests the Mercurial equivalent of the given git command
+    """suggests the Mercurial equivalent of the given git command
 
     Usage: hg githelp -- <git command>
-    '''
+    """
 
     if len(args) == 0 or (len(args) == 1 and args[0] == b'git'):
         raise error.Abort(
@@ -90,11 +89,11 @@ def parseoptions(ui, cmdoptions, args):
             args = fancyopts.fancyopts(list(args), cmdoptions, opts, True)
             break
         except getopt.GetoptError as ex:
-            if r"requires argument" in ex.msg:
+            if "requires argument" in ex.msg:
                 raise
-            if (r'--' + ex.opt) in ex.msg:
+            if ('--' + ex.opt) in ex.msg:
                 flag = b'--' + pycompat.bytestr(ex.opt)
-            elif (r'-' + ex.opt) in ex.msg:
+            elif ('-' + ex.opt) in ex.msg:
                 flag = b'-' + pycompat.bytestr(ex.opt)
             else:
                 raise error.Abort(
@@ -112,18 +111,18 @@ def parseoptions(ui, cmdoptions, args):
 
             ui.warn(_(b"ignoring unknown option %s\n") % flag)
 
-    args = list([convert(x) for x in args])
+    args = [convert(x) for x in args]
     opts = dict(
         [
             (k, convert(v)) if isinstance(v, bytes) else (k, v)
-            for k, v in pycompat.iteritems(opts)
+            for k, v in opts.items()
         ]
     )
 
     return args, opts
 
 
-class Command(object):
+class Command:
     def __init__(self, name):
         self.name = name
         self.args = []
@@ -132,7 +131,7 @@ class Command(object):
     def __bytes__(self):
         cmd = b"hg " + self.name
         if self.opts:
-            for k, values in sorted(pycompat.iteritems(self.opts)):
+            for k, values in sorted(self.opts.items()):
                 for v in values:
                     if v:
                         if isinstance(v, int):
@@ -164,7 +163,7 @@ class Command(object):
         return AndCommand(self, other)
 
 
-class AndCommand(object):
+class AndCommand:
     def __init__(self, left, right):
         self.left = left
         self.right = right
@@ -209,7 +208,7 @@ def add(ui, repo, *args, **kwargs):
 
 def am(ui, repo, *args, **kwargs):
     cmdoptions = []
-    args, opts = parseoptions(ui, cmdoptions, args)
+    parseoptions(ui, cmdoptions, args)
     cmd = Command(b'import')
     ui.status(bytes(cmd), b"\n")
 
@@ -628,8 +627,17 @@ def log(ui, repo, *args, **kwargs):
         (b'', b'stat', None, b''),
         (b'', b'graph', None, b''),
         (b'p', b'patch', None, b''),
+        (b'G', b'grep-diff', b'', b''),
+        (b'S', b'pickaxe-regex', b'', b''),
     ]
     args, opts = parseoptions(ui, cmdoptions, args)
+    grep_pat = opts.get(b'grep_diff') or opts.get(b'pickaxe_regex')
+    if grep_pat:
+        cmd = Command(b'grep')
+        cmd[b'--diff'] = grep_pat
+        ui.status(b'%s\n' % bytes(cmd))
+        return
+
     ui.status(
         _(
             b'note: -v prints the entire commit message like Git does. To '
@@ -1139,7 +1147,7 @@ def svn(ui, repo, *args, **kwargs):
 
 def svndcommit(ui, repo, *args, **kwargs):
     cmdoptions = []
-    args, opts = parseoptions(ui, cmdoptions, args)
+    parseoptions(ui, cmdoptions, args)
 
     cmd = Command(b'push')
 
@@ -1148,7 +1156,7 @@ def svndcommit(ui, repo, *args, **kwargs):
 
 def svnfetch(ui, repo, *args, **kwargs):
     cmdoptions = []
-    args, opts = parseoptions(ui, cmdoptions, args)
+    parseoptions(ui, cmdoptions, args)
 
     cmd = Command(b'pull')
     cmd.append(b'default-push')
@@ -1173,7 +1181,7 @@ def svnrebase(ui, repo, *args, **kwargs):
     cmdoptions = [
         (b'l', b'local', None, b''),
     ]
-    args, opts = parseoptions(ui, cmdoptions, args)
+    parseoptions(ui, cmdoptions, args)
 
     pullcmd = Command(b'pull')
     pullcmd.append(b'default-push')
